#!/usr/bin/perl -w
use strict;
use sigtrap;

# Copyright Philipp L. Wesche 2003
# Released under the terms of the General Public License (GPL)
# The GPL can be found at:
# http://www.gnu.org/copyleft

# An implementation of recent research done at Edinburgh University
# indicating that most words remain readable so long as the first
# and last letter are correct, while the letters in between may be mixed up.

print "\nThis program was inspired by an email message in which a message appears\nin which only the first and last letter of every word\nare in their correct positions. A language with short words,\nsuch as English or Spanish, will generally give the best results!\n\nPlease enter your text below, and confirm by pressing \<ENTER\>\n\n";
my $phrase = <STDIN>;
chomp ($phrase);

my $chunksize = 5;

my @words = split (/[^A-Za-z]/, $phrase);
my @non_alphabetic = split (/[A-Za-z]+/, $phrase);

# remove all zero-length elements
for (my $e = 0; $e < @non_alphabetic; ++$e) {
	unless (length($non_alphabetic[$e]) > 0) {
		splice (@non_alphabetic, $e, 1);
	}
}
for (my $a = 0; $a < @words; ++$a) {
	unless (length($words[$a]) > 0) {
		splice (@words, $a, 1);
	}
}


for (my $kunti = 0; $kunti < @words; ++$kunti) {
	my @newword;
	my @letters = split (//, $words[$kunti]);

	# splice off and keep first and last letters; these are not permutated
	my $first = shift (@letters);
	my $last = pop (@letters);
	my @result;
	my $result_word;

	# permutation only makes sense if there are at least two middle letters
	if (@letters > 1) {
		my @parts;
		my $times = int (@letters / $chunksize);
		if ($times > 1) {
			my $divisor = int (@letters / $times);
			for (my $minicount = 0; $minicount < $times; ++$minicount) {
				my $start = $minicount * $chunksize;
				my $end = (((@letters - 1) < ($start + $chunksize - 1)) ? (@letters - 1) : ($start + $chunksize - 1));
				if ($minicount == $times - 1) {
					$end = @letters - 1;
				}
				push (@parts, join("", @letters[$start..$end]));
			}
		} else {
			@parts = (join("", @letters));
		}
		# then permutate until you get input != output
		undef @result;
		foreach my $var (@parts) {
			my @array;
			do {
				@array = split (//, $var);
				undef @newword;
				while (@array > 0) {
					my $randnum = int rand(@array);
					push (@newword, splice (@array, $randnum, 1));
				}
			} while (join("", @newword) eq $var);
		push (@result, @newword);
		}
	} elsif (defined($letters[0])) {
		$result[0] = $letters[0];
	}
	if (defined ($result[0])) {
		$result_word = $first.join("", @result).$last;
	} elsif (defined ($last)) {
		$result_word = $first.$last;
	} else {
		$result_word = $first;
	}
	print $result_word;
	if (defined ($non_alphabetic[$kunti])) {
		print $non_alphabetic[$kunti];
	}
}
print "\n";
